import asyncio
import logging

from asyncua import Client, ua

_logger = logging.getLogger('asyncua')

# Server URL
url = "opc.tcp://localhost:4870/"

class SubHandler(object):
    """
    Subscription Handler. To receive events from server for a subscription
    data_change and event methods are called directly from receiving thread.
    Do not do expensive, slow or network operation there. Create another
    thread if you need to do such a thing.
    NOTE: Subscription not working with TIA v13 OPC server :( 
    """
    def datachange_notification(self, node, val, data):
        print("New data change event", node, val)

    def event_notification(self, event):
        print("New event", event)


async def main():
    
    async with Client(url=url) as client:
        # Client has a few methods to get proxy to UA nodes that should always be in address space such as Root or Objects
        _logger.info("Root node is: %r", client.nodes.root)
        _logger.info("Objects node is: %r", client.nodes.objects)

        # Node objects have methods to read and write node attributes as well as browse or populate address space
        _logger.info("Children of root are: %r", await client.nodes.root.get_children())
        
        # List all tags
        tags = await client.nodes.root.get_child(["0:Objects", "2:WinCC Panel RT", "4:Tags"])
        _logger.info("\nTags:\n%r\n", await tags.get_children())

        # Get a specific node knowing its node id
        myvar = client.get_node("ns=4;s=SCADA_pec_senzor_temperatura")
        _logger.info(myvar)
        valObj = await myvar.read_data_value() # get value of node as a DataValue object
        _logger.info("Node DataValue object: %r", valObj)
        val = await myvar.read_value() # get value of a node as a Python builtin
        _logger.info("Node value: %r", val)
        _logger.info("Node status code: %r", valObj.StatusCode.name)
        
        # Write to a node
        x = 103.9
        _logger.info("Writing value of %f to a node", x)
        await myvar.write_value(x) # set node value using implicit data type
        await myvar.write_value(ua.Variant(x, ua.VariantType.Float)) #set node value using explicit data type
        # Check the variable
        _logger.info("Node value: %r", await myvar.read_value())

        # Subscribing to a variable node
        # NOT WORKING with TIA v13
        # handler = SubHandler()
        # sub = await client.create_subscription(500, handler)
        # handle = await sub.subscribe_data_change(varPec)
        # await asyncio.sleep(0.1)
        # We can also subscribe to events from server
        # await sub.subscribe_events()
        # await sub.unsubscribe(handle)
        # await sub.delete()

if __name__ == "__main__":
    logging.basicConfig(level=logging.INFO)
    asyncio.run(main())
